Exception: Udb::ConfigNotFoundError

Inherits:
StandardError
  • Object
show all
Defined in:
lib/udb/resolver.rb

Overview

resolves the specification in the context of a config, and writes to a generation folder

Raised by Resolver#cfg_info when a config name or path cannot be found.