Class: Udb::LogicNode::MemoizedState

Inherits:
Object
  • Object
show all
Defined in:
lib/udb/logic.rb

Overview

object to hold results of expensive calculations LogicNode type and children are frozen at construction so we can safely remember and return these values

Instance Attribute Summary collapse

Instance Attribute Details

#cnf_formObject

Returns the value of attribute cnf_form.



1238
1239
1240
# File 'lib/udb/logic.rb', line 1238

def cnf_form
  @cnf_form
end

#equisat_cnfObject

Returns the value of attribute equisat_cnf.



1238
1239
1240
# File 'lib/udb/logic.rb', line 1238

def equisat_cnf
  @equisat_cnf
end

#equiv_cnfObject

Returns the value of attribute equiv_cnf.



1238
1239
1240
# File 'lib/udb/logic.rb', line 1238

def equiv_cnf
  @equiv_cnf
end

#is_cnfObject

Returns the value of attribute is_cnf.



1238
1239
1240
# File 'lib/udb/logic.rb', line 1238

def is_cnf
  @is_cnf
end

#is_nested_cnfObject

Returns the value of attribute is_nested_cnf.



1238
1239
1240
# File 'lib/udb/logic.rb', line 1238

def is_nested_cnf
  @is_nested_cnf
end

#is_reducedObject

Returns the value of attribute is_reduced.



1238
1239
1240
# File 'lib/udb/logic.rb', line 1238

def is_reduced
  @is_reduced
end

#is_satisfiableObject

Returns the value of attribute is_satisfiable.



1238
1239
1240
# File 'lib/udb/logic.rb', line 1238

def is_satisfiable
  @is_satisfiable
end

#literalsObject

Returns the value of attribute literals.



1238
1239
1240
# File 'lib/udb/logic.rb', line 1238

def literals
  @literals
end

#termsObject

Returns the value of attribute terms.



1238
1239
1240
# File 'lib/udb/logic.rb', line 1238

def terms
  @terms
end

#terms_no_antecendentsObject

Returns the value of attribute terms_no_antecendents.



1238
1239
1240
# File 'lib/udb/logic.rb', line 1238

def terms_no_antecendents
  @terms_no_antecendents
end