Class: Udb::Manual
- Inherits:
-
TopLevelDatabaseObject
- Object
- DatabaseObject
- TopLevelDatabaseObject
- Udb::Manual
- Defined in:
- lib/udb/obj/manual.rb
Instance Attribute Summary
Attributes inherited from DatabaseObject
#arch, #data, #data_path, #long_name, #name
Instance Method Summary collapse
-
#marketing_name ⇒ String
The title of the manual, as used by marketing.
-
#repo_path=(path) ⇒ Object
for manuals that reference an external repo, set the url to that repo data (file path).
- #version(name) ⇒ Object
- #versions ⇒ Object
Methods inherited from TopLevelDatabaseObject
create_json_schemer_resolver, #initialize, #key?, #keys, #validate
Methods inherited from DatabaseObject
#<=>, #__source, #cfg_arch, #cfg_arch?, #clone, #defer, #defined_by_condition, #description, #initialize, #inspect, #kind, #source_line
Constructor Details
This class inherits a constructor from Udb::TopLevelDatabaseObject
Instance Method Details
#marketing_name ⇒ String
Returns The title of the manual, as used by marketing.
25 |
# File 'lib/udb/obj/manual.rb', line 25 def marketing_name = @data["marketing_name"] |
#repo_path=(path) ⇒ Object
for manuals that reference an external repo, set the url to that repo data (file path)
28 29 30 31 |
# File 'lib/udb/obj/manual.rb', line 28 def repo_path=(path) @repo_path = Pathname.new(path) versions.each { |v| v.repo_path = @repo_path } end |
#version(name) ⇒ Object
20 21 22 |
# File 'lib/udb/obj/manual.rb', line 20 def version(name) versions.find { |v| v.name == name } end |
#versions ⇒ Object
14 15 16 17 18 |
# File 'lib/udb/obj/manual.rb', line 14 def versions return @versions unless @versions.nil? @versions = @arch.manual_versions.select { |mv| mv.manual == self } end |